@@ -1584,9 +1584,11 @@ get_tags_intersecting_sorted (NautilusBatchRenameDialog *self,
             if (text_changed_mode == TEXT_WAS_DELETED)
             {
                 selection_intersects_tag_start = end_position > tag_data->position &&
-                                                 end_position <= tag_end_position;
+                                                 end_position <= tag_end_position &&
+                                                 start_position <= tag_data->position;
                 selection_intersects_tag_end = start_position >= tag_data->position &&
-                                               start_position < tag_end_position;
+                                               start_position < tag_end_position &&
+                                               end_position >= tag_end_position;
                 tag_is_contained_in_selection = start_position <= tag_data->position &&
                                                 end_position >= tag_end_position;
             }
@@ -1751,7 +1753,7 @@ nautilus_batch_rename_dialog_finalize (GObject *object)
     g_list_free (dialog->listbox_labels_old);
     g_list_free (dialog->listbox_icons);
 
-    for (l = dialog->selection_metadata; l != NULL; l = l->next)
+    for (l = dialog->selection_metadata; l != NULL; l = g_list_delete_link (l, l))
     {
         FileMetadata *file_metadata;
 
