@@ -21,7 +21,6 @@
 
 #pragma once
 
-#include <eel/eel-vfs-extensions.h>
 #include <gio/gio.h>
 #include <nautilus-extension.h>
 
@@ -59,7 +58,7 @@ typedef gint32 RequestCounter[REQUEST_TYPE_LAST];
 #define REQUEST_WANTS_TYPE(request, type) ((request) & (1<<(type)))
 #define REQUEST_SET_TYPE(request, type) (request) |= (1<<(type))
 
-struct NautilusDirectoryDetails
+struct NautilusDirectoryPrivate
 {
 	/* The location. */
 	GFile *location;
