@@ -37,14 +37,11 @@
 char *   nautilus_get_user_directory                 (void);
 char *   nautilus_get_home_directory_uri             (void);
 gboolean nautilus_is_root_directory                  (GFile *dir);
+gboolean nautilus_is_root_for_scheme                 (GFile      *dir,
+                                                      const char *scheme);
 gboolean nautilus_is_home_directory                  (GFile *dir);
 gboolean nautilus_is_home_directory_file             (GFile *dir,
 						      const char *filename);
-gboolean nautilus_is_search_directory                (GFile *dir);
-gboolean nautilus_is_recent_directory                (GFile *dir);
-gboolean nautilus_is_starred_directory              (GFile *dir);
-gboolean nautilus_is_trash_directory                 (GFile *dir);
-gboolean nautilus_is_other_locations_directory       (GFile *dir);
 GMount * nautilus_get_mounted_mount_for_root         (GFile *location);
 
 gboolean nautilus_should_use_templates_directory     (void);
