@@ -5,6 +5,7 @@
  */
 
 #include "nautilus-grid-cell.h"
+#include "nautilus-tag-manager.h"
 
 struct _NautilusGridCell
 {
@@ -39,7 +40,7 @@ update_icon (NautilusGridCell *self)
     item = nautilus_view_cell_get_item (NAUTILUS_VIEW_CELL (self));
     g_return_if_fail (item != NULL);
     file = nautilus_view_item_get_file (item);
-    icon_size = nautilus_view_item_get_icon_size (item);
+    g_object_get (self, "icon-size", &icon_size, NULL);
     scale_factor = gtk_widget_get_scale_factor (GTK_WIDGET (self));
     flags = NAUTILUS_FILE_ICON_FLAGS_USE_THUMBNAILS;
 
@@ -101,10 +102,12 @@ update_emblems (NautilusGridCell *self)
     GtkWidget *child;
     GtkIconTheme *theme;
     g_autolist (GIcon) emblems = NULL;
+    g_autofree gchar *file_uri = NULL;
 
     item = nautilus_view_cell_get_item (NAUTILUS_VIEW_CELL (self));
     g_return_if_fail (item != NULL);
     file = nautilus_view_item_get_file (item);
+    file_uri = nautilus_file_get_uri (file);
 
     /* Remove old emblems. */
     while ((child = gtk_widget_get_first_child (self->emblems_box)) != NULL)
@@ -112,17 +115,17 @@ update_emblems (NautilusGridCell *self)
         gtk_box_remove (GTK_BOX (self->emblems_box), child);
     }
 
+    if (nautilus_tag_manager_file_is_starred (nautilus_tag_manager_get (), file_uri))
+    {
+        gtk_box_append (GTK_BOX (self->emblems_box),
+                        gtk_image_new_from_icon_name ("starred-symbolic"));
+    }
+
     theme = gtk_icon_theme_get_for_display (gdk_display_get_default ());
     emblems = nautilus_file_get_emblem_icons (file);
     for (GList *l = emblems; l != NULL; l = l->next)
     {
-        g_autoptr (GtkIconPaintable) icon_paintable = NULL;
-
-        /* Workaround for gtk_icon_theme_has_gicon() ignoring unthemed icons.
-         * See: https://gitlab.gnome.org/GNOME/nautilus/-/issues/2789 and
-         * https://gitlab.gnome.org/GNOME/gtk/-/issues/5709 */
-        icon_paintable = gtk_icon_theme_lookup_by_gicon (theme, l->data, 16, 1, 0, 0);
-        if (g_strcmp0 (gtk_icon_paintable_get_icon_name (icon_paintable), "image-missing") == 0)
+        if (!gtk_icon_theme_has_gicon (theme, l->data))
         {
             g_autofree gchar *icon_string = g_icon_to_string (l->data);
             g_warning ("Failed to add emblem. “%s” not found in the icon theme",
@@ -135,7 +138,6 @@ update_emblems (NautilusGridCell *self)
     }
 }
 
-
 static void
 on_file_changed (NautilusGridCell *self)
 {
@@ -157,8 +159,16 @@ on_file_changed (NautilusGridCell *self)
 }
 
 static void
-on_item_size_changed (NautilusGridCell *self)
+on_icon_size_changed (NautilusGridCell *self)
 {
+    g_autoptr (NautilusViewItem) item = nautilus_view_cell_get_item (NAUTILUS_VIEW_CELL (self));
+
+    if (item == NULL)
+    {
+        /* Cell is not bound to an item yet. Do nothing. */
+        return;
+    }
+
     update_icon (self);
     update_captions (self);
 }
@@ -202,6 +212,28 @@ on_label_query_tooltip (GtkWidget  *widget,
     return FALSE;
 }
 
+static void
+on_starred_changed (NautilusTagManager *tag_manager,
+                    GList              *changed_files,
+                    gpointer            user_data)
+{
+    NautilusGridCell *self = NAUTILUS_GRID_CELL (user_data);
+    g_autoptr (NautilusViewItem) item = NULL;
+    NautilusFile *file;
+
+    item = nautilus_view_cell_get_item (NAUTILUS_VIEW_CELL (self));
+    if (item == NULL)
+    {
+        return;
+    }
+
+    file = nautilus_view_item_get_file (item);
+    if (g_list_find (changed_files, file))
+    {
+        update_emblems (self);
+    }
+}
+
 static void
 finalize (GObject *object)
 {
@@ -237,13 +269,16 @@ nautilus_grid_cell_init (NautilusGridCell *self)
 {
     gtk_widget_init_template (GTK_WIDGET (self));
 
+    g_signal_connect (self, "notify::icon-size",
+                      G_CALLBACK (on_icon_size_changed), NULL);
     g_signal_connect (self->label, "query-tooltip",
                       G_CALLBACK (on_label_query_tooltip), NULL);
 
+    g_signal_connect_object (nautilus_tag_manager_get (), "starred-changed",
+                             G_CALLBACK (on_starred_changed), self, G_CONNECT_DEFAULT);
+
     /* Connect automatically to an item. */
     self->item_signal_group = g_signal_group_new (NAUTILUS_TYPE_VIEW_ITEM);
-    g_signal_group_connect_swapped (self->item_signal_group, "notify::icon-size",
-                                    (GCallback) on_item_size_changed, self);
     g_signal_group_connect_swapped (self->item_signal_group, "notify::is-cut",
                                     (GCallback) on_item_is_cut_changed, self);
     g_signal_group_connect_swapped (self->item_signal_group, "file-changed",
