@@ -31,6 +31,9 @@ G_DEFINE_TYPE (NautilusGridView, nautilus_grid_view, NAUTILUS_TYPE_LIST_BASE)
 
 static guint get_icon_size_for_zoom_level (NautilusGridZoomLevel zoom_level);
 
+#define get_view_item(li) \
+        (NAUTILUS_VIEW_ITEM (gtk_tree_list_row_get_item (GTK_TREE_LIST_ROW (gtk_list_item_get_item (li)))))
+
 static gint
 nautilus_grid_view_sort (gconstpointer a,
                          gconstpointer b,
@@ -49,6 +52,16 @@ nautilus_grid_view_sort (gconstpointer a,
                                                           self->reversed);
 }
 
+static void
+real_begin_loading (NautilusFilesView *files_view)
+{
+    NautilusGridView *self = NAUTILUS_GRID_VIEW (files_view);
+
+    NAUTILUS_FILES_VIEW_CLASS (nautilus_grid_view_parent_class)->begin_loading (files_view);
+
+    self->directories_first = nautilus_files_view_should_sort_directories_first (NAUTILUS_FILES_VIEW (self));
+}
+
 static void
 real_bump_zoom_level (NautilusFilesView *files_view,
                       int                zoom_increment)
@@ -157,12 +170,11 @@ set_zoom_level (NautilusGridView *self,
     self->zoom_level = new_level;
 
     /* The zoom level may change how many captions are allowed. Update it before
-     * setting the icon size, under the assumption that NautilusGridCell
+     * notifying the icon size change, under the assumption that NautilusGridCell
      * updates captions whenever the icon size is set*/
     set_captions_from_preferences (self);
 
-    nautilus_list_base_set_icon_size (NAUTILUS_LIST_BASE (self),
-                                      get_icon_size_for_zoom_level (new_level));
+    g_object_notify (G_OBJECT (self), "icon-size");
 
     nautilus_files_view_update_toolbar_menus (NAUTILUS_FILES_VIEW (self));
 }
@@ -361,13 +373,14 @@ action_sort_order_changed (GSimpleAction *action,
                            GVariant      *value,
                            gpointer       user_data)
 {
+    g_autoptr (GVariant) old_value = g_action_get_state (G_ACTION (action));
     const gchar *target_name;
     NautilusGridView *self = NAUTILUS_GRID_VIEW (user_data);
     NautilusViewModel *model;
     g_autoptr (GtkCustomSorter) sorter = NULL;
 
     /* Don't resort if the action is in the same state as before */
-    if (g_variant_equal (value, g_action_get_state (G_ACTION (action))))
+    if (g_variant_equal (value, old_value))
     {
         return;
     }
@@ -419,8 +432,7 @@ on_captions_preferences_changed (NautilusGridView *self)
 
     /* Hack: this relies on the assumption that NautilusGridCell updates
      * captions whenever the icon size is set (even if it's the same value). */
-    nautilus_list_base_set_icon_size (NAUTILUS_LIST_BASE (self),
-                                      get_icon_size_for_zoom_level (self->zoom_level));
+    g_object_notify (G_OBJECT (self), "icon-size");
 }
 
 static void
@@ -444,7 +456,7 @@ bind_cell (GtkSignalListItemFactory *factory,
     g_autoptr (NautilusViewItem) item = NULL;
 
     cell = gtk_list_item_get_child (listitem);
-    item = listitem_get_view_item (listitem);
+    item = get_view_item (listitem);
     g_return_if_fail (item != NULL);
 
     nautilus_view_item_set_item_ui (item, cell);
@@ -459,11 +471,6 @@ bind_cell (GtkSignalListItemFactory *factory,
         parent = gtk_widget_get_parent (cell);
         gtk_widget_set_halign (parent, GTK_ALIGN_CENTER);
         gtk_widget_set_valign (parent, GTK_ALIGN_START);
-        gtk_widget_set_margin_top (parent, 3);
-        gtk_widget_set_margin_bottom (parent, 3);
-        gtk_widget_set_margin_start (parent, 3);
-        gtk_widget_set_margin_end (parent, 3);
-
 
         gtk_accessible_update_relation (GTK_ACCESSIBLE (parent),
                                         GTK_ACCESSIBLE_RELATION_LABELLED_BY, cell, NULL,
@@ -478,7 +485,7 @@ unbind_cell (GtkSignalListItemFactory *factory,
 {
     g_autoptr (NautilusViewItem) item = NULL;
 
-    item = listitem_get_view_item (listitem);
+    item = get_view_item (listitem);
 
     /* item may be NULL when row has just been destroyed. */
     if (item != NULL)
@@ -496,9 +503,14 @@ setup_cell (GtkSignalListItemFactory *factory,
     NautilusGridCell *cell;
 
     cell = nautilus_grid_cell_new (NAUTILUS_LIST_BASE (self));
-    setup_cell_common (listitem, NAUTILUS_VIEW_CELL (cell));
+    gtk_list_item_set_child (listitem, GTK_WIDGET (cell));
+    setup_cell_common (G_OBJECT (listitem), NAUTILUS_VIEW_CELL (cell));
     setup_cell_hover (NAUTILUS_VIEW_CELL (cell));
 
+    g_object_bind_property (self, "icon-size",
+                            cell, "icon-size",
+                            G_BINDING_SYNC_CREATE);
+
     nautilus_grid_cell_set_caption_attributes (cell, self->caption_attributes);
 }
 
@@ -527,6 +539,7 @@ create_view_ui (NautilusGridView *self)
     gtk_grid_view_set_single_click_activate (GTK_GRID_VIEW (widget), FALSE);
     gtk_grid_view_set_max_columns (GTK_GRID_VIEW (widget), 20);
     gtk_grid_view_set_enable_rubberband (GTK_GRID_VIEW (widget), TRUE);
+    gtk_grid_view_set_tab_behavior (GTK_GRID_VIEW (widget), GTK_LIST_TAB_ITEM);
 
     /* While we don't want to use GTK's click activation, we'll let it handle
      * the key activation part (with Enter).
@@ -552,6 +565,7 @@ nautilus_grid_view_class_init (NautilusGridViewClass *klass)
     object_class->dispose = dispose;
     object_class->finalize = finalize;
 
+    files_view_class->begin_loading = real_begin_loading;
     files_view_class->bump_zoom_level = real_bump_zoom_level;
     files_view_class->can_zoom_in = real_can_zoom_in;
     files_view_class->can_zoom_out = real_can_zoom_out;
