@@ -31,6 +31,7 @@
 #include "nautilus-file.h"
 #include "nautilus-file-utilities.h"
 #include "nautilus-global-preferences.h"
+#include "nautilus-scheme.h"
 #include "nautilus-search-directory.h"
 #include "nautilus-search-popover.h"
 #include "nautilus-mime-actions.h"
@@ -49,7 +50,7 @@ struct _NautilusQueryEditor
     GtkWidget *mime_types_tag;
     GtkWidget *date_range_tag;
 
-    guint search_changed_timeout_id;
+    guint search_changed_idle_id;
     gboolean change_frozen;
 
     GFile *location;
@@ -80,12 +81,6 @@ static void nautilus_query_editor_changed (NautilusQueryEditor *editor);
 
 G_DEFINE_TYPE (NautilusQueryEditor, nautilus_query_editor, GTK_TYPE_WIDGET);
 
-/* A hunt-and-peck typist types at 25-35 words per minute, which means 342 to 480ms between strokes.
- * An average touch typist types at 50-70 wpm, which means 171 to 240ms "under ideal conditions".
- * A 150ms default search triggering delay is too short even for fast typists in general,
- * so wait 400ms after typing, to improve performance by not spamming search engines: */
-#define SEARCH_CHANGED_TIMEOUT 400
-
 static void
 update_fts_sensitivity (NautilusQueryEditor *editor)
 {
@@ -93,14 +88,10 @@ update_fts_sensitivity (NautilusQueryEditor *editor)
 
     if (editor->location)
     {
-        g_autoptr (NautilusFile) file = NULL;
-        g_autofree gchar *uri = NULL;
-
-        file = nautilus_file_get (editor->location);
-        uri = g_file_get_uri (editor->location);
+        g_autoptr (NautilusFile) file = nautilus_file_get (editor->location);
 
         fts_sensitive = !nautilus_file_is_other_locations (file) &&
-                        !g_str_has_prefix (uri, "network://") &&
+                        !g_file_has_uri_scheme (editor->location, SCHEME_NETWORK) &&
                         !(nautilus_file_is_remote (file) &&
                           location_settings_search_get_recursive_for_location (editor->location) == NAUTILUS_QUERY_RECURSIVE_NEVER);
         nautilus_search_popover_set_fts_sensitive (NAUTILUS_SEARCH_POPOVER (editor->popover),
@@ -138,7 +129,7 @@ nautilus_query_editor_dispose (GObject *object)
 
     editor = NAUTILUS_QUERY_EDITOR (object);
 
-    g_clear_handle_id (&editor->search_changed_timeout_id, g_source_remove);
+    g_clear_handle_id (&editor->search_changed_idle_id, g_source_remove);
 
     gtk_widget_unparent (gtk_widget_get_first_child (GTK_WIDGET (editor)));
     g_clear_pointer (&editor->tags_box, gtk_widget_unparent);
@@ -171,6 +162,15 @@ nautilus_query_editor_grab_focus (GtkWidget *widget)
     return FALSE;
 }
 
+void
+nautilus_query_editor_select_all_text (NautilusQueryEditor *editor)
+{
+    if (gtk_widget_get_visible (GTK_WIDGET (editor)))
+    {
+        gtk_widget_grab_focus (editor->text);
+    }
+}
+
 static void
 nautilus_query_editor_get_property (GObject    *object,
                                     guint       prop_id,
@@ -183,12 +183,6 @@ nautilus_query_editor_get_property (GObject    *object,
 
     switch (prop_id)
     {
-        case PROP_LOCATION:
-        {
-            g_value_set_object (value, editor->location);
-        }
-        break;
-
         case PROP_QUERY:
         {
             g_value_set_object (value, editor->query);
@@ -302,15 +296,14 @@ nautilus_query_editor_class_init (NautilusQueryEditorClass *class)
     /**
      * NautilusQueryEditor::location:
      *
-     * The current location of the query editor.
+     * Binding target for the slot's location. To be applied to the existing
+     * query, or when creating a new one.
      */
     g_object_class_install_property (gobject_class,
                                      PROP_LOCATION,
-                                     g_param_spec_object ("location",
-                                                          "Location of the search",
-                                                          "The current location of the editor",
+                                     g_param_spec_object ("location", NULL, NULL,
                                                           G_TYPE_FILE,
-                                                          G_PARAM_READWRITE));
+                                                          G_PARAM_WRITABLE | G_PARAM_STATIC_STRINGS));
 
     /**
      * NautilusQueryEditor::query:
@@ -331,14 +324,6 @@ nautilus_query_editor_class_init (NautilusQueryEditorClass *class)
     gtk_widget_class_set_accessible_role (widget_class, GTK_ACCESSIBLE_ROLE_SEARCH_BOX);
 }
 
-GFile *
-nautilus_query_editor_get_location (NautilusQueryEditor *editor)
-{
-    g_return_val_if_fail (NAUTILUS_IS_QUERY_EDITOR (editor), NULL);
-
-    return g_object_ref (editor->location);
-}
-
 static void
 create_query (NautilusQueryEditor *editor)
 {
@@ -384,7 +369,7 @@ entry_changed_internal (NautilusQueryEditor *editor)
     const gchar *text = gtk_editable_get_text (GTK_EDITABLE (editor->text));
     gboolean is_empty = (text == NULL && *text == '\0');
 
-    editor->search_changed_timeout_id = 0;
+    editor->search_changed_idle_id = 0;
 
     gtk_widget_set_child_visible (editor->clear_icon, !is_empty);
 
@@ -407,15 +392,15 @@ static void
 entry_changed_cb (GtkWidget           *entry,
                   NautilusQueryEditor *editor)
 {
+    g_clear_handle_id (&editor->search_changed_idle_id, g_source_remove);
+
     if (editor->change_frozen)
     {
         return;
     }
 
-    g_clear_handle_id (&editor->search_changed_timeout_id, g_source_remove);
-    editor->search_changed_timeout_id = g_timeout_add (SEARCH_CHANGED_TIMEOUT,
-                                                       G_SOURCE_FUNC (entry_changed_internal),
-                                                       editor);
+    editor->search_changed_idle_id = g_idle_add (G_SOURCE_FUNC (entry_changed_internal),
+                                                 editor);
 }
 
 static GtkWidget *
@@ -657,6 +642,7 @@ nautilus_query_editor_init (NautilusQueryEditor *editor)
     /* setup the filter menu button */
     editor->dropdown_button = gtk_menu_button_new ();
     gtk_menu_button_set_icon_name (GTK_MENU_BUTTON (editor->dropdown_button), "funnel-symbolic");
+    gtk_widget_set_tooltip_text (GTK_WIDGET (editor->dropdown_button), _("Filter Search Results"));
     gtk_menu_button_set_popover (GTK_MENU_BUTTON (editor->dropdown_button), editor->popover);
     gtk_widget_set_parent (editor->dropdown_button, GTK_WIDGET (editor));
     gtk_widget_add_css_class (editor->dropdown_button, "circular");
@@ -710,7 +696,6 @@ nautilus_query_editor_set_location (NautilusQueryEditor *editor,
                                     GFile               *location)
 {
     g_autoptr (NautilusDirectory) directory = NULL;
-    NautilusDirectory *base_model;
     gboolean should_notify;
 
     g_return_if_fail (NAUTILUS_IS_QUERY_EDITOR (editor));
@@ -722,10 +707,10 @@ nautilus_query_editor_set_location (NautilusQueryEditor *editor,
     directory = nautilus_directory_get (location);
     if (NAUTILUS_IS_SEARCH_DIRECTORY (directory))
     {
+        NautilusSearchDirectory *search = NAUTILUS_SEARCH_DIRECTORY (directory);
         g_autoptr (GFile) real_location = NULL;
 
-        base_model = nautilus_search_directory_get_base_model (NAUTILUS_SEARCH_DIRECTORY (directory));
-        real_location = nautilus_directory_get_location (base_model);
+        real_location = nautilus_query_get_location (nautilus_search_directory_get_query (search));
 
         should_notify = g_set_object (&editor->location, real_location);
     }
