@@ -27,6 +27,8 @@ struct _NautilusViewCellPrivate
     NautilusListBase *view; /* Unowned */
     NautilusViewItem *item; /* Owned reference */
 
+    guint icon_size;
+
     gboolean called_once;
 };
 
@@ -38,6 +40,7 @@ enum
     PROP_0,
     PROP_VIEW,
     PROP_ITEM,
+    PROP_ICON_SIZE,
     N_PROPS
 };
 
@@ -66,6 +69,12 @@ nautilus_view_cell_get_property (GObject    *object,
         }
         break;
 
+        case PROP_ICON_SIZE:
+        {
+            g_value_set_uint (value, priv->icon_size);
+        }
+        break;
+
         default:
         {
             G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -96,6 +105,12 @@ nautilus_view_cell_set_property (GObject      *object,
         }
         break;
 
+        case PROP_ICON_SIZE:
+        {
+            priv->icon_size = g_value_get_uint (value);
+        }
+        break;
+
         default:
         {
             G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -138,6 +153,12 @@ nautilus_view_cell_class_init (NautilusViewCellClass *klass)
                                                  "", "",
                                                  NAUTILUS_TYPE_VIEW_ITEM,
                                                  G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+    properties[PROP_ICON_SIZE] = g_param_spec_uint ("icon-size",
+                                                    "", "",
+                                                    NAUTILUS_LIST_ICON_SIZE_SMALL,
+                                                    NAUTILUS_GRID_ICON_SIZE_EXTRA_LARGE,
+                                                    NAUTILUS_GRID_ICON_SIZE_LARGE,
+                                                    G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
     g_object_class_install_properties (object_class, N_PROPS, properties);
 }
