@@ -26,8 +26,6 @@
 
 #include "nautilus-window.h"
 
-#include <eel/eel-debug.h>
-#include <eel/eel-vfs-extensions.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <gdk/gdkkeysyms.h>
 #include <glib/gi18n.h>
@@ -63,6 +61,8 @@
 #include "nautilus-module.h"
 #include "nautilus-pathbar.h"
 #include "nautilus-profile.h"
+#include "nautilus-progress-indicator.h"
+#include "nautilus-scheme.h"
 #include "nautilus-signaller.h"
 #include "nautilus-toolbar.h"
 #include "nautilus-trash-monitor.h"
@@ -96,6 +96,9 @@ struct _NautilusWindow
 {
     AdwApplicationWindow parent_instance;
 
+    GtkWidget *app_button;
+    GMenuModel *undo_redo_section;
+
     AdwTabView *tab_view;
     AdwTabBar *tab_bar;
     AdwTabPage *menu_page;
@@ -103,7 +106,7 @@ struct _NautilusWindow
     GList *slots;
     NautilusWindowSlot *active_slot; /* weak reference */
 
-    GtkWidget *content_flap;
+    GtkWidget *split_view;
 
     /* Side Pane */
     GtkWidget *places_sidebar;     /* the actual GtkPlacesSidebar */
@@ -228,7 +231,7 @@ action_go_starred (GSimpleAction *action,
     g_autoptr (GFile) starred = NULL;
 
     window = NAUTILUS_WINDOW (user_data);
-    starred = g_file_new_for_uri ("starred:///");
+    starred = g_file_new_for_uri (SCHEME_STARRED ":///");
 
     nautilus_window_open_location_full (window, starred, 0, NULL, NULL);
 }
@@ -574,7 +577,6 @@ location_to_tooltip (GBinding           *binding,
                      NautilusWindowSlot *slot)
 {
     GFile *location = g_value_get_object (input);
-    g_autofree gchar *location_name = NULL;
 
     if (location == NULL)
     {
@@ -584,14 +586,14 @@ location_to_tooltip (GBinding           *binding,
     /* Set the tooltip on the label's parent (the tab label hbox),
      * so it covers all of the tab label.
      */
-    location_name = g_file_get_parse_name (location);
 
-    if (eel_uri_is_search (location_name))
+    if (g_file_has_uri_scheme (location, SCHEME_SEARCH))
     {
         g_value_set_string (output, nautilus_window_slot_get_title (slot));
     }
     else
     {
+        g_autofree gchar *location_name = g_file_get_parse_name (location);
         g_value_set_string (output, location_name);
     }
 
@@ -753,7 +755,6 @@ nautilus_window_new_tab (NautilusWindow *window)
     NautilusWindowSlot *current_slot;
     AdwTabPage *page;
     GFile *location;
-    g_autofree gchar *uri = NULL;
 
     page = get_current_page (window);
     current_slot = NAUTILUS_WINDOW_SLOT (adw_tab_page_get_child (page));
@@ -761,8 +762,7 @@ nautilus_window_new_tab (NautilusWindow *window)
 
     if (location != NULL)
     {
-        uri = g_file_get_uri (location);
-        if (eel_uri_is_search (uri))
+        if (g_file_has_uri_scheme (location, SCHEME_SEARCH))
         {
             location = g_file_new_for_path (g_get_home_dir ());
         }
@@ -901,7 +901,7 @@ places_sidebar_show_other_locations_with_flags (NautilusWindow             *wind
 {
     GFile *location;
 
-    location = g_file_new_for_uri ("other-locations:///");
+    location = g_file_new_for_uri (SCHEME_OTHER_LOCATIONS ":///");
 
     open_location_cb (window, location, open_flags);
 
@@ -914,7 +914,7 @@ places_sidebar_show_starred_location (NautilusWindow             *window,
 {
     GFile *location;
 
-    location = g_file_new_for_uri ("starred:///");
+    location = g_file_new_for_uri (SCHEME_STARRED ":///");
 
     open_location_cb (window, location, open_flags);
 
@@ -999,7 +999,7 @@ action_restore_tab (GSimpleAction *action,
 
     data = g_queue_pop_head (window->tab_data_queue);
 
-    location = nautilus_file_get_location (data->file);
+    location = nautilus_bookmark_get_location (data->current_location_bookmark);
 
     slot = nautilus_window_create_and_init_slot (window, flags);
 
@@ -1017,8 +1017,8 @@ action_toggle_sidebar (GSimpleAction *action,
     NautilusWindow *window = NAUTILUS_WINDOW (user_data);
     gboolean revealed;
 
-    revealed = adw_flap_get_reveal_flap (ADW_FLAP (window->content_flap));
-    adw_flap_set_reveal_flap (ADW_FLAP (window->content_flap), !revealed);
+    revealed = adw_overlay_split_view_get_show_sidebar (ADW_OVERLAY_SPLIT_VIEW (window->split_view));
+    adw_overlay_split_view_set_show_sidebar (ADW_OVERLAY_SPLIT_VIEW (window->split_view), !revealed);
 }
 
 
@@ -1184,15 +1184,15 @@ toast_undo_deleted_get_label (NautilusFileUndoInfo *undo_info)
     if (length == 1)
     {
         file_label = g_file_get_basename (files->data);
-        /* Translators: only one item has been deleted and %s is its name. */
+        /* Translators: only one item has been moved to trash and %s is its name. */
         label = g_markup_printf_escaped (_("“%s” moved to trash"), file_label);
         g_free (file_label);
     }
     else
     {
-        /* Translators: one or more items might have been deleted, and %d
+        /* Translators: one or more items might have been moved to trash, and %d
          * is the count. */
-        label = g_markup_printf_escaped (ngettext ("%d file deleted", "%d files deleted", length), length);
+        label = g_markup_printf_escaped (ngettext ("%d file moved to trash", "%d files moved to trash", length), length);
     }
 
     return label;
@@ -1225,6 +1225,66 @@ toast_undo_unstar_get_label (NautilusFileUndoInfo *undo_info)
     return label;
 }
 
+static void
+update_undo_redo_menu_items (NautilusWindow               *window,
+                             NautilusFileUndoInfo         *info,
+                             NautilusFileUndoManagerState  undo_state)
+{
+    gboolean undo_active;
+    gboolean redo_active;
+    g_autofree gchar *undo_label = NULL;
+    g_autofree gchar *redo_label = NULL;
+    g_autofree gchar *undo_description = NULL;
+    g_autofree gchar *redo_description = NULL;
+    gboolean is_undo;
+    g_autoptr (GMenu) updated_section = g_menu_new ();
+    g_autoptr (GMenuItem) undo_menu_item = NULL;
+    g_autoptr (GMenuItem) redo_menu_item = NULL;
+    GAction *action;
+
+    /* Look at the last action from the undo manager, and get the text that
+     * describes it, e.g. "Undo Create Folder"/"Redo Create Folder"
+     */
+    undo_active = redo_active = FALSE;
+    if (info != NULL && undo_state > NAUTILUS_FILE_UNDO_MANAGER_STATE_NONE)
+    {
+        is_undo = undo_state == NAUTILUS_FILE_UNDO_MANAGER_STATE_UNDO;
+
+        /* The last action can either be undone/redone. Activate the corresponding
+         * menu item and deactivate the other
+         */
+        undo_active = is_undo;
+        redo_active = !is_undo;
+        nautilus_file_undo_info_get_strings (info, &undo_label, &undo_description,
+                                             &redo_label, &redo_description);
+    }
+
+    /* Set the label of the undo and redo menu items, and activate them appropriately
+     */
+    if (!undo_active || undo_label == NULL)
+    {
+        g_free (undo_label);
+        undo_label = g_strdup (_("_Undo"));
+    }
+    undo_menu_item = g_menu_item_new (undo_label, "win.undo");
+    g_menu_append_item (updated_section, undo_menu_item);
+    action = g_action_map_lookup_action (G_ACTION_MAP (window), "undo");
+    g_simple_action_set_enabled (G_SIMPLE_ACTION (action), undo_active);
+
+    if (!redo_active || redo_label == NULL)
+    {
+        g_free (redo_label);
+        redo_label = g_strdup (_("_Redo"));
+    }
+    redo_menu_item = g_menu_item_new (redo_label, "win.redo");
+    g_menu_append_item (updated_section, redo_menu_item);
+    action = g_action_map_lookup_action (G_ACTION_MAP (window), "redo");
+    g_simple_action_set_enabled (G_SIMPLE_ACTION (action), redo_active);
+
+    nautilus_gmenu_set_from_model (G_MENU (window->undo_redo_section),
+                                   G_MENU_MODEL (updated_section));
+}
+
 static void
 nautilus_window_on_undo_changed (NautilusFileUndoManager *manager,
                                  NautilusWindow          *window)
@@ -1236,6 +1296,8 @@ nautilus_window_on_undo_changed (NautilusFileUndoManager *manager,
     undo_info = nautilus_file_undo_manager_get_action ();
     state = nautilus_file_undo_manager_get_state ();
 
+    update_undo_redo_menu_items (window, undo_info, state);
+
     if (undo_info != NULL &&
         state == NAUTILUS_FILE_UNDO_MANAGER_STATE_UNDO)
     {
@@ -1267,7 +1329,7 @@ nautilus_window_on_undo_changed (NautilusFileUndoManager *manager,
             /* Don't pop up a notification if the focus is not in the this
              * window. This is an easy way to know from which window was the
              * unstart operation made */
-            if (eel_uri_is_starred (g_file_get_uri (location)) &&
+            if (g_file_has_uri_scheme (location, SCHEME_STARRED) &&
                 gtk_window_is_active (GTK_WINDOW (window)) &&
                 !nautilus_file_undo_info_starred_is_starred (NAUTILUS_FILE_UNDO_INFO_STARRED (undo_info)))
             {
@@ -1313,6 +1375,7 @@ nautilus_window_show_operation_notification (NautilusWindow *window,
 
     toast = adw_toast_new (main_label);
     adw_toast_set_priority (toast, ADW_TOAST_PRIORITY_HIGH);
+    adw_toast_set_use_markup (toast, FALSE);
 
     current_location = nautilus_window_slot_get_location (window->active_slot);
     if (gtk_window_is_active (GTK_WINDOW (window)))
@@ -1351,14 +1414,6 @@ on_path_bar_open_location (NautilusWindow    *window,
     }
 }
 
-GtkWidget *
-nautilus_window_get_toolbar (NautilusWindow *window)
-{
-    g_return_val_if_fail (NAUTILUS_IS_WINDOW (window), NULL);
-
-    return window->toolbar;
-}
-
 static void
 setup_toolbar (NautilusWindow *window)
 {
@@ -1496,18 +1551,12 @@ extra_drag_value_cb (AdwTabBar    *self,
                 action = nautilus_dnd_get_preferred_action (file, G_FILE (file_list->data));
             }
         }
-        else if (G_VALUE_HOLDS (value, G_TYPE_STRING))
+        else if (G_VALUE_HOLDS (value, G_TYPE_STRING) || G_VALUE_HOLDS (value, GDK_TYPE_TEXTURE))
         {
             action = GDK_ACTION_COPY;
         }
     }
 
-    /* We set the preferred action on the drop from the results of this function,
-     * but since we don't have access to the GtkDropTarget, we can't get the preferred
-     * action in ::drop, so let's set it as data on the page. We probably should
-     * expose the preferred action within libadwaita.
-     */
-    g_object_set_data (G_OBJECT (page), "drag-action", GINT_TO_POINTER (action));
     return action;
 }
 
@@ -1520,7 +1569,7 @@ extra_drag_drop_cb (AdwTabBar    *self,
     NautilusWindowSlot *slot = NAUTILUS_WINDOW_SLOT (adw_tab_page_get_child (page));
     NautilusFilesView *view = NAUTILUS_FILES_VIEW (nautilus_window_slot_get_current_view (slot));
     GFile *target_location = nautilus_window_slot_get_location (slot);
-    GdkDragAction action = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (page), "drag-action"));
+    GdkDragAction action = adw_tab_bar_get_extra_drag_preferred_action (self);
 
     return nautilus_dnd_perform_drop (view, value, action, target_location);
 }
@@ -1556,6 +1605,18 @@ const GActionEntry win_entries[] =
     { "toggle-sidebar", action_toggle_sidebar },
 };
 
+static void
+window_set_back_forward_accelerators (void)
+{
+    GApplication *app = g_application_get_default ();
+    gboolean ltr = (gtk_widget_get_default_direction () == GTK_TEXT_DIR_LTR);
+
+#define ACCELS(...) ((const char *[]) { __VA_ARGS__, NULL })
+
+    nautilus_application_set_accelerators (app, "win.back", ACCELS (ltr ? "<alt>Left" : "<alt>Right", "Back"));
+    nautilus_application_set_accelerators (app, "win.forward", ACCELS (ltr ? "<alt>Right" : "<alt>Left", "Forward"));
+}
+
 static void
 nautilus_window_initialize_actions (NautilusWindow *window)
 {
@@ -1569,11 +1630,8 @@ nautilus_window_initialize_actions (NautilusWindow *window)
                                      win_entries, G_N_ELEMENTS (win_entries),
                                      window);
 
-#define ACCELS(...) ((const char *[]) { __VA_ARGS__, NULL })
-
     app = g_application_get_default ();
-    nautilus_application_set_accelerators (app, "win.back", ACCELS ("<alt>Left", "Back"));
-    nautilus_application_set_accelerators (app, "win.forward", ACCELS ("<alt>Right", "Forward"));
+    window_set_back_forward_accelerators ();
     nautilus_application_set_accelerators (app, "win.enter-location", ACCELS ("<control>l", "Go", "OpenURL"));
     nautilus_application_set_accelerator (app, "win.new-tab", "<control>t");
     nautilus_application_set_accelerator (app, "win.close-current-view", "<control>w");
@@ -1609,7 +1667,7 @@ nautilus_window_initialize_actions (NautilusWindow *window)
 #undef ACCELS
 
     action = g_action_map_lookup_action (G_ACTION_MAP (window), "toggle-sidebar");
-    g_object_bind_property (window->content_flap, "folded",
+    g_object_bind_property (window->split_view, "collapsed",
                             action, "enabled", G_BINDING_SYNC_CREATE);
 }
 
@@ -1637,17 +1695,22 @@ nautilus_window_constructed (GObject *self)
 
     setup_tab_view (window);
 
-    /* Lets not support GDK_ACTION_LINK for now.  This only impacts x11
-     * and on x11 we are using a hack in list-base to handle GDK_ACTION_LINK
-     * which we can not replicate here because we don't have access to the
-     * GtkDropTarget to check the actions on the GdkDrag.
+    /* Only allow tab DnD in Wayland.  We are using a hack in list-base to
+     * get the preferred action which we can not replicate here because we
+     * don't have access to the GtkDropTarget to check the actions on the GdkDrag.
+     * See: https://gitlab.gnome.org/GNOME/gtk/-/merge_requests/4982
      */
-    adw_tab_bar_setup_extra_drop_target (window->tab_bar,
-                                         GDK_ACTION_COPY | GDK_ACTION_MOVE,
-                                         (GType [2]) {GDK_TYPE_FILE_LIST, G_TYPE_STRING}, 2);
-    adw_tab_bar_set_extra_drag_preload (window->tab_bar, TRUE);
-    g_signal_connect (window->tab_bar, "extra-drag-value", G_CALLBACK (extra_drag_value_cb), NULL);
-    g_signal_connect (window->tab_bar, "extra-drag-drop", G_CALLBACK (extra_drag_drop_cb), NULL);
+#ifdef GDK_WINDOWING_WAYLAND
+    if (GDK_IS_WAYLAND_DISPLAY (gtk_widget_get_display (GTK_WIDGET (window))))
+    {
+        adw_tab_bar_setup_extra_drop_target (window->tab_bar,
+                                             GDK_ACTION_COPY | GDK_ACTION_MOVE,
+                                             (GType [3]) {GDK_TYPE_TEXTURE, GDK_TYPE_FILE_LIST, G_TYPE_STRING}, 3);
+        adw_tab_bar_set_extra_drag_preload (window->tab_bar, TRUE);
+        g_signal_connect (window->tab_bar, "extra-drag-value", G_CALLBACK (extra_drag_value_cb), NULL);
+        g_signal_connect (window->tab_bar, "extra-drag-drop", G_CALLBACK (extra_drag_drop_cb), NULL);
+    }
+#endif
 
     nautilus_window_set_up_sidebar (window);
 
@@ -1665,8 +1728,6 @@ nautilus_window_constructed (GObject *self)
                                                     G_CALLBACK (nautilus_window_sync_bookmarks),
                                                     window, G_CONNECT_SWAPPED);
 
-    nautilus_toolbar_on_window_constructed (NAUTILUS_TOOLBAR (window->toolbar));
-
     nautilus_profile_end (NULL);
 }
 
@@ -2061,6 +2122,7 @@ nautilus_window_back_or_forward_in_new_tab (NautilusWindow              *window,
             state->forward_list = g_list_prepend (state->forward_list, state->current_location_bookmark);
             state->current_location_bookmark = state->back_list->data;
             state->back_list = state->back_list->next;
+            g_clear_object (&state->current_search_query);
         }
         break;
 
@@ -2069,6 +2131,7 @@ nautilus_window_back_or_forward_in_new_tab (NautilusWindow              *window,
             state->back_list = g_list_prepend (state->back_list, state->current_location_bookmark);
             state->current_location_bookmark = state->forward_list->data;
             state->forward_list = state->forward_list->next;
+            g_clear_object (&state->current_search_query);
         }
         break;
 
@@ -2158,6 +2221,7 @@ nautilus_window_init (NautilusWindow *window)
 
     g_type_ensure (NAUTILUS_TYPE_TOOLBAR);
     g_type_ensure (NAUTILUS_TYPE_GTK_PLACES_SIDEBAR);
+    g_type_ensure (NAUTILUS_TYPE_PROGRESS_INDICATOR);
     gtk_widget_init_template (GTK_WIDGET (window));
 
     g_signal_connect_object (window->places_sidebar,
@@ -2293,13 +2357,17 @@ nautilus_window_class_init (NautilusWindowClass *class)
 
     gtk_widget_class_set_template_from_resource (wclass,
                                                  "/org/gnome/nautilus/ui/nautilus-window.ui");
+    gtk_widget_class_bind_template_child (wclass, NautilusWindow, app_button);
+    gtk_widget_class_bind_template_child (wclass, NautilusWindow, undo_redo_section);
     gtk_widget_class_bind_template_child (wclass, NautilusWindow, toolbar);
-    gtk_widget_class_bind_template_child (wclass, NautilusWindow, content_flap);
+    gtk_widget_class_bind_template_child (wclass, NautilusWindow, split_view);
     gtk_widget_class_bind_template_child (wclass, NautilusWindow, places_sidebar);
     gtk_widget_class_bind_template_child (wclass, NautilusWindow, toast_overlay);
     gtk_widget_class_bind_template_child (wclass, NautilusWindow, tab_view);
     gtk_widget_class_bind_template_child (wclass, NautilusWindow, tab_bar);
 
+    gtk_widget_class_bind_template_callback (wclass, window_set_back_forward_accelerators);
+
     signals[SLOT_ADDED] =
         g_signal_new ("slot-added",
                       G_TYPE_FROM_CLASS (class),
@@ -2414,3 +2482,16 @@ nautilus_window_search (NautilusWindow *window,
         g_warning ("Trying search on a slot but no active slot present");
     }
 }
+
+gboolean
+nautilus_window_is_menu_visible (NautilusWindow *self)
+{
+    GtkWidget *menu;
+
+    g_return_val_if_fail (NAUTILUS_IS_WINDOW (self), FALSE);
+
+    menu = GTK_WIDGET (gtk_menu_button_get_popover (GTK_MENU_BUTTON (self->app_button)));
+    g_return_val_if_fail (menu != NULL, FALSE);
+
+    return gtk_widget_is_visible (menu);
+}
